unit SerInt;
Interface

 const     RFehler: Byte=0;
           ReceiveFlag: Boolean=False;
           BufferOverFlow: Boolean=False;

 procedure ComInit( ComNr, Baud :Byte; ShortTimeout, LongTimeout: Word);
 {Bit/s 0:300 1:600 2:1200 3:2400 4:4800 5:9600}
 procedure ComDone;
 procedure SendChar(Ch :Char);
 procedure SendString(S :String);
 function  ReceiveByte: Byte;
 procedure ReceiveDummy;
 procedure SetTimeout(Timeout :Word);
 function  ReceiveAvail :Boolean;
Implementation
 uses      Dos,Crt;

 const     RBufferMax    =10000;
           RLongTimeout  :Word=100;
           RShortTimeout :Word=50;
           RTimeout      :Word=RShortTimeout;
           Ringcount     :Word=0;
           RCnt          :Word=0;
           SBase         :Word=$02F8;
           IBase         :Byte=$0020;
           RFlag         :Boolean=False;

  var      IMR,OCW2,OCW3,IntEnable,ModemCtrl,
           RecBuffer,TransReg,LineStatus :Word;
           LineCtrlReg,BaudloReg,BaudhiReg :Word;
           LineCtrlSave,ModemCtrlSave,IntCtrlSave,
           BaudloSave,BaudhiSave :Byte;
           IntNum,MaskBit,UnMaskBit,OldImrMask,
           Baudlo,Baudhi :Byte;
           TimerP,TimerPOld,SeriellP,SeriellPOld :Pointer;
           RTimeoutFlag :Boolean;
           TmrHsTimeout,TimerCount :Longint;
           RBuffer :Array[1..RBufferMax] of Byte;


 procedure DeInstall(VektorNr :Integer; P :Pointer);
 begin
  asm cli
  end;
  SetIntVec(VektorNr,P);
  asm sti
  end;
 end;

 procedure Install(VektorNr :Integer; PNew :Pointer;var POld :Pointer);
 begin
  asm cli
  end;
  GetIntVec(VektorNr,POld);
  SetIntVec(VektorNr,PNew);
  asm sti
  end;
 end;

 procedure JmpRoutine(Adress :Pointer);
  inline($5B/
         $58/
         $87/$5E/$0E/
         $87/$46/$10/
         $89/$EC/
         $5D/
         $07/
         $1F/
         $5F/
         $5E/
         $5A/
         $59/
         $CB);

 procedure TimerInterrupt(BP :Word); interrupt;
 begin
  Inc(TimerCount);
  JmpRoutine(TimerPOld);
 end;

 procedure TimerInit;
 begin
  TimerCount:=0;
  TimerP:=@TimerInterrupt;
  Install(8,TimerP,TimerPOld);
 end;

 procedure TimerDone;
 begin
  DeInstall(8,TimerPOld);
 end;

 procedure TimerHsReset(TimerHs :Longint);
 var R :Real;
 begin
  R:=TimerHs;
  R:=R/5.55;
  TmrHsTimeout:=Round(R);
  TimerCount:=0;
  TimerInit;
 end;

 function TimerOverFlow :Boolean;
 var TOF :Boolean;
 begin
  TOF:=(TmrHsTimeout<=TimerCount);
  if TOF then TimerCount:=0;
  TimerOverFlow:=TOF;
 end;

 procedure ReceiveInt(BP :Word); interrupt;
 var B,Status :Byte;
 begin
  asm
   mov dx,Linestatus;
  @1:
   jmp @3;
  @3:
   in al,dx;
   mov bl,al;
   and al,01h;
   jz @1;
   jmp @2;
  @2:
   and bl,0Eh;
   mov Status,bl;
   mov dx,RecBuffer;
   in al,dx;
   mov B,al;
  end;
  if Status <> 0 then RFehler:=Status and $0F;
  Inc(RingCount);
  if RingCount=RCnt then BufferOverFlow:=true;
  if RingCount>RBufferMax then RingCount:=1;
  RBuffer[RingCount]:=B;
  asm
   mov dx,OCW2;
   mov al,20h;
   out dx,al;
  end;
 end;

 procedure ComInit(ComNr,Baud :Byte; ShortTimeout,LongTimeout :Word);
 var regs :Registers;
 begin
  if ShortTimeout > 0 then RShortTimeout:=ShortTimeout div 10;
  if LongTimeout > 0 then RLongTimeout:=LongTimeout div 10;
  if ComNr = 1 then
  begin
   SBase:=$3F8;
   UnMaskBit:=$EF;
   MaskBit:=$10;
   IntNum:=$0C;
  end
 else
  begin
   SBase:=$2F8;
   UnMaskBit:=$F7;
   MaskBit:=$08;
   IntNum:=$0B;
  end;
  RecBuffer:=SBase;
  TransReg:=SBase;
  LineStatus:=SBase+5;
  LineCtrlReg:=SBase+3;
  BaudloReg:=SBase;
  BaudhiReg:=SBase+1;
  case Baud of
   0: begin Baudhi:=1;Baudlo:=$80;end;
   1: begin Baudhi:=0;Baudlo:=$C0;end;
   2: begin Baudhi:=0;Baudlo:=$60;end;
   3: begin Baudhi:=0;Baudlo:=$30;end;
   4: begin Baudhi:=0;Baudlo:=$18;end;
  else
   begin
    Baudhi:=0;
    Baudlo:=$0C;
   end;
  end;
  IMR:=IBase+1;
  OCW3:=IBase;
  OCW2:=IBase;
  SeriellP:=@ReceiveInt;
  Install(IntNum,SeriellP,SeriellPOld);
  IntEnable:=SBase+1;
  ModemCtrl:=SBase+4;
  asm
   mov dx,LineCtrlReg;
   in al,dx;
   mov LineCtrlSave,al;
   mov al,$80;
   out dx,al;
   jmp @4;
  @4:
   mov dx,BaudhiReg;
   in al,dx;
   mov BaudhiSave,al;
   mov al,Baudhi;
   out dx,al;
   jmp @5
  @5:
   mov dx,BaudloReg;
   in al,dx
   mov BaudloSave,al;
   mov al,Baudlo;
   out dx,al;
   jmp @6
  @6:
   mov dx,LineCtrlReg;
   mov al,$03;
   out dx,al;
   jmp @7
  @7:
   mov dx,ModemCtrl;
   in al,dx;
   mov ModemCtrlSave,al
   mov al,0Bh;
   out dx,al;
   jmp @2
  @2:
   mov dx,IntEnable;
   in al,dx
   mov IntCtrlSave,al;
   mov al,0;
   out dx,al;
  end;
  asm
   mov dx,IMR;
   in al,dx;
   mov OldImrMask,al;
   and al,UnMaskBit;
   jmp @1
  @1:
   out dx,al;
  end;
  asm
   mov dx,LineCtrlReg;
   mov al,$80;
   out dx,al;
   jmp @4
  @4:
   mov dx,BaudhiReg;
   mov al,Baudhi;
   out dx,al;
   jmp @5
  @5:
   mov dx,BaudloReg;
   mov al,Baudlo;
   out dx,al;
   jmp @6
  @6:
   mov dx,LineCtrlReg;
   mov al,$03;
   out dx,al;
   jmp @7
  @7:
   mov dx,ModemCtrl;
   mov al,0Bh;
   out dx,al;
   jmp @2
  @2:
   mov dx,IntEnable;
   mov al,1;
   out dx,al;
  end;
  delay(100);
  ReceiveDummy;
 end;

 procedure ComDone;
 begin
  asm
   cli;
   mov bl,OldImrMask;
   and bl,MaskBit;
   mov dx,IMR;
   in al,dx;
   or al,bl;
   jmp @1
  @1:
   jmp @2
  @2:
   out dx,al;
   mov dx,LineCtrlReg;
   mov al,$80;
   out dx,al;
   jmp @4
  @4:
   mov dx,BaudhiReg;
   mov al,BaudhiSave;
   out dx,al;
   jmp @5
  @5:
   mov dx,BaudloReg;
   mov al,BaudloSave;
   out dx,al;
   jmp @6
  @6:
   mov dx,LineCtrlReg;
   mov al,LineCtrlSave;
   out dx,al;
   jmp @7
  @7:
   mov dx,ModemCtrl;
   mov al,MOdemCtrlSave;
   out dx,al;
   jmp @8
  @8:
   mov dx,IntEnable;
   mov al,IntCtrlSave;
   out dx,al;
  end;
  DeInstall(IntNum,SeriellPOld);
  asm sti;
  end;
 end;

  procedure SendChar(Ch :Char);
  var Ch1 :Char;
  begin
   ch1:=ch;
   asm
    mov dx,LineStatus;
   @1:
    jmp @3
   @3:
    in al,dx;
    and al,020h;
    jz @1;
    mov dx,Transreg;
    mov al,ch1;
    jmp @2
   @2:
    out dx,al;
   end;
  end;

 procedure SendString(S :String);
 var i,l :Integer;
 begin
  l:=Length(S);
  i:=1;
  while (I<=L) do
  begin
   SendChar(S[I]);
   Inc(I);
  end;
 end;

 function ReceiveByte :Byte;
 var regs :Registers;
 ReceiveStatus :Byte;
 Cnt :Longint;
 begin
  RFlag:=false;
  if RCnt <> RingCount then
  begin
   RFlag:=true;
   Inc(RCnt);
   if RCnt >RBufferMax then RCnt:=1;
  end
 else
  begin
   TimerHsReset(RTimeout);
   RTimeoutFlag:=false;
   repeat
    if RCnt <> RingCount then
    begin
     RFlag:=true;
     Inc(RCnt);
     if RCnt>RBufferMax then RCnt:=1;
    end
   else
    begin
     if TimerOverFlow then
     begin
      RTimeoutFlag:=true;
     end;
    end;
   until (RFlag) or (RTimeoutFlag);
   TimerDone;
  end;
  ReceiveFlag:=RFlag;
  if RFlag then ReceiveByte:=RBuffer[RCnt]
 else
  ReceiveByte:=0;
 end;

 procedure ReceiveDummy;
 var Dummy :Byte;
     I :Word;
 begin
  I:=0;
  RTimeout:=RShortTimeout;
  repeat
   Dummy:=ReceiveByte;
   Inc(I);
  until ((NOT (RFlag)) or (I>=10000));
  RTimeout:=RLongTimeout;
 end;

 procedure SetTimeout(Timeout :Word);
 begin
  if Timeout > 0 then RTimeout:=Timeout div 10
 else
  RTimeout:=RLongTimeout;
 end;

 function ReceiveAvail :Boolean;
 var Flag :Boolean;
 begin
  flag:=(not(RCnt=RingCount) or BufferOverFlow);
  ReceiveAvail:=Flag;
 end;


begin
end.

--- Yuppie v2.10
 * Origin: Angus oi! (2:246/806.222)


